#!/bin/sh

ISO_PATH=$1

echo "Schneider Electric Data Center Expert USB Drive Preparation Script v1.0b (Linux)"

if [ "$EUID" -ne 0 ]
then
    echo "ERROR: You must run this script with root permissions."
    exit 1
fi

if [ -z $ISO_PATH ]
then
    echo "ERROR: You must supply the path & filename of the .iso image."
    exit 1;
fi

if [ ! -f $ISO_PATH ]
then
    echo "ERROR: File '$ISO_PATH' does not exist.  You must supply the path & filename of the .iso image."
    exit 1;
fi

ISO_FILENAME=`basename $ISO_PATH`

if [ ! -f /lib/ld-linux.so.2 ]
then
    echo "This utility requires 32-bit libraries."
    echo "Try running 'yum install glibc.i686' or 'dnf install glibc.i686'"
    exit 1
fi

echo "Enter the device name of your USB Flash key: "
read DEVICE_NAME

if [ -z $DEVICE_NAME ]
then
    echo "ERROR: You must enter a device name to continue."
    exit 1
fi

if [ ! -b $DEVICE_NAME ]
then
   echo "ERROR: $DEVICE_NAME does not appear to be a valid device name."
   exit 1
fi

DEVICE_OK=0

for i in /dev/disk/by-id/usb* ; do
    DEV_NAME=`readlink -f $i`
    if [ $DEV_NAME == $DEVICE_NAME ]
    then
        DEVICE_OK=1
    fi
done

if [ $DEVICE_OK == 0 ]
then
    echo "ERROR: $DEVICE_NAME does not appear to be a USB device name."
    exit 1
fi

echo "Using device $DEVICE_NAME"

grep -q $DEVICE_NAME /proc/mounts
GREP_RESULT=$?
if [ $GREP_RESULT -eq 0 ]
then
    echo "ERROR: Device $DEVICE_NAME is already mounted.  Please unmount this device before running this script."
    exit 1
fi

while [ -z $CONTINUE_FORMAT ]
do
    echo "This device will be formatted, and all data will be lost.  Are you are you want to continue? [y/n]"
    read CONTINUE_FORMAT
done

if [ $CONTINUE_FORMAT != "y" ]
then
    exit 1
fi

if [ -x /sbin/sfdisk ]
then
    SFDISK=/sbin/sfdisk
else 
    if [ -x /usr/sbin/sfdisk ]
    then
        SFDISK=/usr/sbin/sfdisk
    else 
        echo "The sfdisk utility needs to be installed in order to run this utility."
        exit 1
    fi
fi

# Detect device size and output size
DISK_SIZE_KB=`$SFDISK -s $DEVICE_NAME`
DISK_SIZE_MB=$(( $(( $DISK_SIZE_KB / 1024 )) - 1))

echo "Creating partition on $DEVICE_NAME, $DISK_SIZE_MB MB"

# Partition device to utilize all space
$SFDISK --force $DEVICE_NAME <<EOF
,,b,*
EOF

partprobe $DEVICE_NAME

if [ -x /sbin/mkfs.fat ]
then
    MKFS=/sbin/mkfs.fat
else 
    if [ -x /usr/sbin/mkfs.fat ]
    then
        MKFS=/usr/sbin/mkfs.fat
    else 
        echo "The mkfs.fat utility needs to be installed in order to run this utility."
        exit 1
    fi
fi

# Create MS-DOS filesystem
DEV_PARTITION=$DEVICE_NAME\1
echo "$MKFS $DEV_PARTITION"
$MKFS -F 32 -n ISXC $DEV_PARTITION

# Mount .iso to get ks.cfg
echo "Fetching configuration files from .iso..."
mkdir ./isoimg
mount -o loop $ISO_PATH ./isoimg
MOUNT_RESULT=$?
if [ $MOUNT_RESULT -ne 0 ]
then
    echo "ERROR: Unable to mount .iso image.  Please ensure this is a a valid ISO 9660 image, and try again."
    exit 1
fi
cp ./isoimg/ks.cfg ./ks.cfg
cp ./isoimg/isolinux/isolinux.cfg ./syslinux.cfg
cp ./isoimg/isolinux/boot.cat ./
cp ./isoimg/isolinux/boot.msg ./
cp ./isoimg/isolinux/initrd.img ./
cp ./isoimg/isolinux/splash.lss ./
cp ./isoimg/isolinux/vmlinuz ./

# Umount .iso
umount ./isoimg
rmdir ./isoimg

# Create folder for usb partition
echo "Creating temporary working folder..."
mkdir ./usb
USB_MOUNT_RESULT=1
while [ $USB_MOUNT_RESULT -ne 0 ]
do
    mount $DEV_PARTITION ./usb >> /dev/null
    USB_MOUNT_RESULT=$?
    if [ $USB_MOUNT_RESULT -ne 0 ]
    then
        echo "Unable to mount usb partition.  Trying again in 1 second..."
        sleep 1
    fi
done

# Copy files to device
echo "Copying files to device $DEV_PARTITION..."
mkdir ./usb/iso
mkdir ./usb/syslinux
cp -f $ISO_PATH ./usb/iso/
mv -f ./boot.cat ./usb/syslinux/
mv -f ./boot.msg ./usb/syslinux/
mv -f ./initrd.img ./usb/syslinux/
mv -f ./splash.lss ./usb/syslinux/
mv -f ./vmlinuz ./usb/syslinux/

# Edit syslinux.cfg to use USB device with a hard-drive install method
sed s/cdrom/hd:LABEL=ISXC/ ./syslinux.cfg > ./usb/syslinux/syslinux.cfg
rm -f ./syslinux.cfg

# Edit ks.cfg to perform hard-drive install method & copy to USB device
sed -e "0,/cdrom/s/cdrom/harddrive --partition=LABEL=ISXC --dir=\/iso\/$ISO_FILENAME/" -e 's/^reboot.*/reboot/' -e '/^%post/,/^%end/ { /^%end/ d }' ./ks.cfg > ./ks.cfg.new

sed -e '$ a\ \nchvt 1\necho -e "\\a" > /dev/tty1\necho -e "\\n\\nSystem restore complete.\\n\\n" > /dev/tty1\nsleep 1\necho -e "\\a" > /dev/tty1\necho -e "Press enter to reboot your StruxureWare Data Center Expert server.\\n\\nRemove your USB flash key when the system reboots.\\n\\n" > /dev/tty1\nread\necho -e "System Rebooting..." > /dev/tty1\n\n%end\n' -e '/^%post/ i %post --nochroot --log=/mnt/sysimage/var/log/anaconda-post-nochroot.log\nls -l /run/install/repo\ncp -r /run/install/repo/NetBotz/upgrade /mnt/sysimage/etc/nbc/\nfind /mnt/sysimage/etc/nbc/upgrade -iname "TRANS.TBL" -delete\n%end\n' -e '/^    # edit for automated/,/^    find \/etc/ { d }' -e '/^#clear screen/,/^fi/ { d }' ./ks.cfg.new > ks.cfg.usb
rm -f ./ks.cfg.new
mv -f ./ks.cfg.usb ./usb/ks.cfg

# Remove old ks.cfg
rm -f ./ks.cfg

# Check and change /tmp permissions, since syslinux needs sticky bit set in /tmp
TMP_PERMS=`stat --format=%a /tmp`
if [ $TMP_PERMS != "1777" ]; then
	echo "Incorrect permissions set on the /tmp directory.  Would you like to correct the permissions? (y/n)"
	read CORRECT_PERMS
	if [ $CORRECT_PERMS == "y" ]; then
		chmod 1777 /tmp
	fi
fi

# Install syslinux
./deps/syslinux/syslinux -if $DEV_PARTITION

RESULT_CODE=$?
if [ $RESULT_CODE -ne 0 ]; then
	echo "FATAL: An error was encountered installing the bootloader."
fi

# Unmount USB device
echo "Cleaning up..."
umount ./usb
rmdir ./usb

# Finished
echo "Done!"
